from id import Id
from instrument import Instrument
from license import License
from method import Method
from name import Name
from pages import Pages
from person import Person
from process_step import ProcessStep
from property import Property
from reference import Reference
from scalar import Scalar
from source import Source
from software import Software
from value import Value
